/*************************************************************************\
    Copyright (c) 2010 Corporation for National Research Initiatives;
                        All rights reserved.
     The CNRI open source license for this software is available at 
                  http://hdl.handle.net/1895.26/1014
---------------------------------------------------------------------------
Developer: Sean Reilly <sreilly@cnri.reston.va.us>
\*************************************************************************/

package net.cnri.dobj;

import net.cnri.cert.*;

import java.security.*;
import java.security.cert.*;
import javax.net.ssl.*;
import java.util.*;



/** This object is used to determine whether the other side of a secure socket 
  * connection is authenticated or not based on their Handle and Digital Object
  * authentication.
  */
public class DOSSLTrustManager 
  implements X509TrustManager
{

  private static boolean TRUST_DEBUG = false;
  
  private ArrayList<X509Certificate> acceptedIssuers = new ArrayList<X509Certificate>();
  private Hashtable cachedPubkeys = new Hashtable();
  
  public DOSSLTrustManager() {
  }
  
  public void checkClientTrusted(X509Certificate chain[], String authType) 
  throws CertificateException
  {
    verifyCertificate(chain);
  }
  public void checkServerTrusted(X509Certificate chain[], String authType) 
  throws CertificateException
  {
    verifyCertificate(chain);
  }
  
  
  private void verifyCertificate(X509Certificate chain[]) 
    throws CertificateException
  {
    if(TRUST_DEBUG) System.err.println("Verifying certificate chain: "+chain);
    if(chain==null || chain.length<=0) {
      throw new CertificateException("No client certificate provided");
    }
    
    int numCerts = 0;
    try {
      for(int i=0; i<chain.length; i++) {
        X509Certificate cert = chain[i];
        if(TRUST_DEBUG) System.err.println("  checking cert "+(i+1)+" of "+chain.length);

        if(cert==null) throw new CertificateException("Broken certificate chain at index "+i);
        
        cert.checkValidity();
        
        CertUtil.verifyCertByResolving(DOClient.getResolver().getResolver(), cert);
        numCerts++;
      }
    } catch (Exception e) {
      if(e instanceof CertificateException) throw (CertificateException)e;
      throw new CertificateException("Error verifying certificate: "+e);
    }
    
    if(numCerts<=0) {
      throw new CertificateException("No certificates provided by peer");
    }
  }
  
  public X509Certificate[] getAcceptedIssuers() {
    if(TRUST_DEBUG) System.err.println("returning issuer list: "+acceptedIssuers);
    return acceptedIssuers.toArray(new X509Certificate[acceptedIssuers.size()]);
  }

  
}


